package com.ym.mec.auth.web.controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.auth.core.service.AccessTokenService;
import com.ym.mec.auth.service.SysUserService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.security.AuthUser;
import com.ym.mec.common.security.SecurityUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.security.core.Authentication;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.provider.token.DefaultTokenServices;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Base64;
import java.util.Map;

@RestController
@Api(tags = "认证服务")
public class TokenController extends BaseController {

	@Autowired
	private DefaultTokenServices tokenService;

	@Autowired
	private RestTemplate restTemplate;

	@Autowired
	private ObjectMapper objectMapper;
	
	@Autowired
	private AccessTokenService accessTokenService;

	@Autowired
	private SysUserService userService;
    
    @PostMapping("/smsLogin")
	@ApiOperation(value = "短信验证码的方式登录")
	@ApiImplicitParams({ @ApiImplicitParam(name = "smsCode", value = "验证码", required = true, dataType = "String"),
			@ApiImplicitParam(name = "clientSecret", value = "固定传 app", required = true, dataType = "String"),
			@ApiImplicitParam(name = "clientId", value = "固定传 app", required = true, dataType = "String"),
			@ApiImplicitParam(name = "phone", value = "手机号", required = true, dataType = "String") })
    public Object smsLogin(HttpServletRequest request, HttpServletResponse response, Authentication authentication) {
    	
    	String clientId = request.getParameter("clientId");
		String clientSecret = request.getParameter("clientSecret");
		if (StringUtils.isBlank(clientId) || StringUtils.isBlank(clientSecret)) {
			return failed("没有指定客户端或秘钥");
		}
		
        return succeed(accessTokenService.getAccessToken(clientId, clientSecret, authentication));
    }

    @GetMapping("/queryUserInfo")
    public SysUser queryUserInfo() {
		AuthUser authUser = SecurityUtils.getUser();
		if(authUser != null){
			return userService.get(authUser.getUserId());
		}
		return null;
    }

	@ApiOperation(value = "获取用户信息")
    @GetMapping("/api/queryUserInfo")
    public Object apiQueryUserInfo() {
		AuthUser authUser = SecurityUtils.getUser();
		if(authUser != null){
			return succeed(userService.get(authUser.getUserId()));
		}
		return failed("获取用户信息失败");
    }
    
    @PostMapping("/usernameLogin")
	@ApiOperation(value = "手机号密码方式登录")
	@ApiImplicitParams({ @ApiImplicitParam(name = "password", value = "密码", required = true, dataType = "String"),
			@ApiImplicitParam(name = "clientSecret", value = "固定传 app", required = true, dataType = "String"),
			@ApiImplicitParam(name = "clientId", value = "固定传 app", required = true, dataType = "String"),
			@ApiImplicitParam(name = "username", value = "手机号", required = true, dataType = "String") })
    public Object usernameLogin(HttpServletRequest request, HttpServletResponse response, Authentication authentication) {
    	
    	String clientId = request.getParameter("clientId");
		String clientSecret = request.getParameter("clientSecret");
		if (StringUtils.isBlank(clientId) || StringUtils.isBlank(clientSecret)) {
			return failed("没有指定客户端或秘钥");
		}
		
        return succeed(accessTokenService.getAccessToken(clientId, clientSecret, authentication));
    }

	/*@PostMapping(value = "/loginIn")
	//@PreAuthorize("hasAnyAuthority('sys_user_manage')")
	public HttpResponseResult login(String username, String password, String clientId, String clientSecret) throws IOException {
		String url = "http://auth-server/oauth/token";

		String base64ClientCredentials = Base64.getEncoder().encodeToString((clientId + ":" + clientSecret).getBytes());

		HttpHeaders headers = new HttpHeaders();
		headers.add("Authorization", "Basic " + base64ClientCredentials);
		headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);

		MultiValueMap<String, String> params = new LinkedMultiValueMap<String, String>();
		params.add("username", username);
		params.add("password", password);
		params.add("grant_type", "password");

		HttpEntity<MultiValueMap<String, String>> entity = new HttpEntity<MultiValueMap<String, String>>(params, headers);

		ResponseEntity<String> resp = restTemplate.exchange(url, HttpMethod.POST, entity, String.class);

		return succeed(objectMapper.readValue(resp.getBody(), Map.class));
	}*/

	@PostMapping(value = "/refreshToken")
	@ApiOperation(value = "刷新token")
	public HttpResponseResult refreshToken(String refreshToken, String clientId, String clientSecret) throws IOException {
		String url = "http://auth-server/oauth/token";

		String base64ClientCredentials = Base64.getEncoder().encodeToString((clientId + ":" + clientSecret).getBytes());

		HttpHeaders headers = new HttpHeaders();
		headers.add("Authorization", "Basic " + base64ClientCredentials);
		headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);

		MultiValueMap<String, String> params = new LinkedMultiValueMap<String, String>();
		params.add("refresh_token", refreshToken);
		params.add("grant_type", "refresh_token");

		HttpEntity<MultiValueMap<String, String>> entity = new HttpEntity<MultiValueMap<String, String>>(params, headers);

		ResponseEntity<String> resp = restTemplate.exchange(url, HttpMethod.POST, entity, String.class);

		return succeed(objectMapper.readValue(resp.getBody(), Map.class));
	}

	@PostMapping(value = "exit")
	@ApiOperation(value = "推出登录")
	public HttpResponseResult logout(@RequestHeader(value = HttpHeaders.AUTHORIZATION) String authHeader) {
		if (StringUtils.isBlank(authHeader)) {
			return failed("退出失败，token 为空");
		}

		String tokenValue = authHeader.replace(OAuth2AccessToken.BEARER_TYPE, StringUtils.EMPTY).trim();

		tokenService.revokeToken(tokenValue);

		return succeed("退出成功");
	}

}
